@@ -6,7 +6,8 @@
 ##    A cross platform perl printer interface                             ##
 ##    This code is made available under the perl artistic licence         ##
 ##                                                                        ##
-##    Documentation is at the end (search for __END__)                    ##
+##    Documentation is at the end (search for __END__) or process with    ##
+##    pod2man/pod2text/pod2html                                           ##
 ##                                                                        ##
 ##    Debugging and code contributions from:                              ##
 ##    David W Phillips (ss0300@dfa.state.ny.us)                           ##
@@ -15,15 +16,13 @@
 ############################################################################
 
 package Printer;
-$VERSION = '0.94b';
+$VERSION = '0.95';
 
 use English;
 use strict;
 no strict 'refs';
 use Carp qw(croak cluck);
-use vars qw(%Env @ISA @EXPORT);
-require Exporter;
-@ISA = qw(Exporter);
+use vars qw(%Env @ISA);
 
 # load environment variables which contain the default printer name (Linux)
 # (from the lprng lpr command manpage)
@@ -32,8 +31,6 @@ use Env qw(PRINTER LPDEST NPRINTER NGPRINTER TEMP PATH);
 
 # macperl $OSNAME is /^macos/i; 
 
-# export the list_printers function
-@EXPORT = qw(list_printers);
 #############################################################################
 sub new {
     # constructor
@@ -41,7 +38,7 @@ sub new {
     my %params = @_;
     my $self = {};
 
-    $self->{'system'} = $OSNAME;
+    $self->{system} = $OSNAME;
 
     # frob the system value to use linux routines below for the
     # various unices
@@ -51,65 +48,85 @@ sub new {
 				 machten next   openbsd dec_osf
 				 svr4    sco_sv unicos  unicosmk
 				 solaris sunos) ) {
-	$self->{'system'} = 'linux';
+	$self->{system} = 'linux';
 
 	# search PATH for lpr, lpq, lp, lpstat (use first found)            DWP
-    my %progs;                                        # will hold prg locs  DWP
-    my @PathDirs = grep {/^[^\.]/} (split /:/,$PATH); # paths, no ./..      DWP
-    foreach my $dir ( @PathDirs ) {                   # go down path        DWP
-        foreach my $prg ( qw(lpr lpq lp lpstat) ) {   # check each prg      DWP
-            next if exists $progs{$prg};              # skip if found       DWP
-            my $loc = "$dir/$prg";                    #                     DWP
-            -f $loc && -x $loc && ($progs{$prg}=$loc);# save location       DWP
-        }                                                                 # DWP
-    }                                                                     # DWP
-    $self->{'program'} = \%progs;                     # include locs in obj DWP
-
+	my %progs;                                        # will hold prg locs  DWP
+	my @PathDirs = grep {/^[^\.]/} (split /:/,$PATH); # paths, no ./..      DWP
+	foreach my $dir ( @PathDirs ) {                   # go down path        DWP
+	    foreach my $prg ( qw(lpr lpq lp lpstat) ) {   # check each prg      DWP
+		next if exists $progs{$prg};              # skip if found       DWP
+		my $loc = "$dir/$prg";                    #                     DWP
+		-f $loc && -x $loc && ($progs{$prg}=$loc);# save location       DWP
+		}                                                                 # DWP
+	}                                                                     # DWP
+	$self->{'program'} = \%progs;                     # include locs in obj DWP
+	
     }
     
 
-    # load system specific modules 
+    # load system specific modules
     BEGIN {
-	if ($self->{'system'} eq "MSWin32") {
+	if ($^O eq "MSWin32") {
+	    # win32 specific modules
 	    require Win32::Registry;  # to list printers
-	    require Win32::AdminMisc; # to find out the windows version
+	    require Win32;
 	}
     }
 
-    $self->{'printer'} = \%params;
+    $self->{printer} = \%params;
 
     # die with an informative message if using an unsupported platform.
-    unless ($self->{'system'} eq 'linux' or $self->{'system'} eq 'MSWin32') {
-      Carp::croak "Platform $OSNAME is not yet supported. Share and enjoy.";
-	return undef;
-    }
+    unless ($self->{system} eq 'linux' or $self->{system} eq 'MSWin32') {
+	Carp::croak "Platform $OSNAME is not yet supported. Share and enjoy.";
+	  return undef;
+      }
     return bless $self, $type;
 
+}
+############################################################################
+sub print_command {
+    # allow users to specify a print command to use for a system
+    my $self = shift();
+    my %systems = @_;
+    my %final_data;
+    
+    foreach my $system (keys %systems) {
+	foreach my $opt (keys %{ $systems{$system} }) {
+	    my %cmd_data = %{ $systems{$system} };
+	    $final_data{$system} = \%cmd_data;
+	}
+    }
+    
+    $self->{print_command} = \%final_data;
+    
 }
 ############################################################################
 sub list_printers {
     # list available printers
-    my $self = shift;
+    my $self = shift();
     my %printers;
 
+
     # linuxish systems
-    if ($self->{'system'} eq "linux") {
+    if ($self->{system} eq "linux") {
         my @prs;
         if ( -f '/etc/printcap' ) {                                         
             # DWP - linux, dec_osf
             open (PRINTCAP, '</etc/printcap') or 
-              Carp::croak "Can't read /etc/printcap: $!";
+		Carp::croak "Can't read /etc/printcap: $!";
             while (<PRINTCAP>) {
-                if ($ARG =~ /\|/) {
+                if ($ARG =~ /^\w/) {
                     chomp $ARG;
-                    $ARG =~ s/\|.*//;
+                    $ARG =~ s!\\!!;
+		    $ARG =~ s!|.*!!;
                     push @prs, $ARG;
                 }
             }
         } elsif ( -f '/etc/printers.conf' ) {                               
             # DWP - solaris
             open (PRINTCNF, '</etc/printers.conf') or                       
-              Carp::croak "Can't read /etc/printers.conf: $!";              
+		Carp::croak "Can't read /etc/printers.conf: $!";              
             while (<PRINTCNF>) {                                            
                 if ($ARG =~ /\|/ or $ARG =~ /^[^:]+:\\/) {                  
                     chomp $ARG;                                             
@@ -120,7 +137,7 @@ sub list_printers {
         } elsif ( -d '/etc/lp/member' ) {                                   
             # DWP - hpux
             opendir (LPMEM, '/etc/lp/member') or                            
-              Carp::croak "Can't readdir /etc/lp/member: $!";               
+		Carp::croak "Can't readdir /etc/lp/member: $!";               
             @prs = grep { /^[^\.]/ && -f "/etc/lp/member/$_" } readdir(LPMEM);
         }                                                                   
         $printers{name} = [ @prs ];
@@ -128,21 +145,21 @@ sub list_printers {
     } # end linux
 
     # win32
-    elsif ($self->{'system'} eq "MSWin32") {
-	# look at registry to get printer names for local machine
+    elsif ($self->{system} eq "MSWin32") {
+       	# look at registry to get printer names for local machine
 	my $Register = 'SYSTEM\CurrentControlSet\Control\Print\Printers';
 	my ($hkey, @key_list, @names, @ports);
 	
 	my $HKEY_LOCAL_MACHINE = $main::HKEY_LOCAL_MACHINE;
 	
 	$HKEY_LOCAL_MACHINE->Open($Register, $hkey) or 
-	  Carp::croak "Can't open registry key $Register: $!";
+	    Carp::croak "Can't open registry key $Register: $!";
 	$hkey->GetKeys(\@key_list);
 	foreach my $key (@key_list) {
 	    my $path = $Register . "\\$key";
 	    my ($pkey, %values, $printers);
 	    $HKEY_LOCAL_MACHINE->Open($path, $pkey) or 
-	      Carp::croak "Can\'t open registry key $path: $!";
+		Carp::croak "Can\'t open registry key $path: $!";
 	    $pkey->GetValues(\%values);
 	    push @ports, $values{Port}[2];
 	    push @names, $values{Name}[2];
@@ -171,31 +188,31 @@ sub use_default {
             # DWP - lpstat -d
 	    my @lpd = grep { /system default destination/i } <LPDEST>;
 	    if ( @lpd == 0 ) {                                        
-	      Carp::cluck 'I can\'t determine your default printer, setting it to lp';
-		$self->{'printer'}{$OSNAME} = "lp";                            
-	    } elsif ( $lpd[-1] =~ /no system default destination/i ) {     
-	      Carp::cluck 'No default printer specified, setting it to lp';    
-		$self->{'printer'}{$OSNAME} = "lp";                        
-	    } elsif ( $lpd[-1] =~ /system default destination:\s*(\S+)/i ) {
-		$self->{'printer'}{$OSNAME} = $1;
-	    } 
+		Carp::cluck 'I can\'t determine your default printer, setting it to lp';
+		  $self->{'printer'}{$OSNAME} = "lp";                            
+	      } elsif ( $lpd[-1] =~ /no system default destination/i ) {     
+		  Carp::cluck 'No default printer specified, setting it to lp';    
+		    $self->{'printer'}{$OSNAME} = "lp";                        
+		} elsif ( $lpd[-1] =~ /system default destination:\s*(\S+)/i ) {
+		    $self->{'printer'}{$OSNAME} = $1;
+		} 
 	} else {
 	    cluck 'I can\'t determine your default printer, setting it to lp'; 
 	    $self->{'printer'}{$OSNAME} = "lp";
 	}
-	print "Linuxish default = $self->{'printer'}{$OSNAME}\n\n";
+	print "Linuxish default = $self->{printer}{$OSNAME}\n\n";
         # DWP - test
     } # end linux
 
     # windows
-    elsif ($self->{'system'} eq "MSWin32") {
+    elsif ($self->{system} eq "MSWin32") {
 	# default name is the human readable printer name (not port)
 	# look in the registry to find it
 	my $register = 'Config\0001\SYSTEM\CurrentControlSet\Control\Print\Printers';
 	my ($hkey, %values);
 	my $HKEY_LOCAL_MACHINE = $main::HKEY_LOCAL_MACHINE;
 	$HKEY_LOCAL_MACHINE->Open($register, $hkey) or 
-	  Carp::croak "Can't open registry key $register: $!";
+	    Carp::croak "Can't open registry key $register: $!";
 	$hkey->GetValues(\%values);
 	my $default = $values{Default}[2];
 	
@@ -204,7 +221,7 @@ sub use_default {
 	my $register = 'SYSTEM\CurrentControlSet\control\Print\Printers';
 	my $path = $register . $default;
 	$HKEY_LOCAL_MACHINE->Open($path, $hkey) or 
-	  Carp::croak "Can't open registry key $path: $!";
+	    Carp::croak "Can't open registry key $path: $!";
 	$hkey->GetValues(\%values);
 	$self->{'printer'}{$OSNAME} = $values{Port}[2];
     } # end win32
@@ -230,45 +247,94 @@ sub print {
 
     # linuxish systems
     if ($self->{'system'} eq "linux") {
-	# use available print program, lpr preferred      # DWP
-        my $lpcmd;                                     # DWP
-        if ( exists $self->{'program'}{'lpr'} ) {      # DWP
-            $lpcmd = $self->{'program'}{'lpr'}.' -P'   # DWP
-        } elsif ( exists $self->{'program'}{'lp'} ) {  # DWP
-            $lpcmd = $self->{'program'}{'lp'}.' -d'    # DWP
-        } else {                                       # DWP
-            Carp::croak "Can't find lpr or lp program for print function" # DWP
-        }                                              # DWP
-    open PRINTER, "| $lpcmd$self->{'printer'}{$OSNAME}" # DWP- use $lpcmd for lpr/lp
-	or Carp::croak "Can't open printer connection to $self->{'printer'}{$OSNAME}: $!";
-	print PRINTER $data;
-	close PRINTER;
-    }
+	# use standard print command
+	unless ($self->{print_command}) {
+	    # use available print program, lpr preferred      # DWP
+	    my $lpcmd;                                     # DWP
+	    if ( exists $self->{'program'}{'lpr'} ) {      # DWP
+		$lpcmd = $self->{'program'}{'lpr'}.' -P'   # DWP
+		} elsif ( exists $self->{'program'}{'lp'} ) {  # DWP
+		    $lpcmd = $self->{'program'}{'lp'}.' -d'    # DWP
+		    } else {                                       # DWP
+			Carp::croak "Can't find lpr or lp program for print function" # DWP
+			}                                              # DWP
+	    open PRINTER, "| $lpcmd$self->{'printer'}{$OSNAME}" # DWP- use $lpcmd for lpr/lp
+		or Carp::croak "Can't open printer connection to $self->{'printer'}{$OSNAME}: $!";
+	    print PRINTER $data;
+	    close PRINTER;
+	} else {
+	    # user has specified a custom print command
+	    if ($self->{print_command}->{linux}->{type} eq 'pipe') {
+		# command accepts piped data
+		open PRINTER, "| $self->{print_command}->{linux}->{command}"
+		    or Carp::croak "Can't open printer connection to $self->{print_command}->{linux}->{command}";
+		print PRINTER $data;
+		close PRINTER;
+	    } else {
+		# command accepts file data, not piped
+		
+		# write $data to a temp file
+		my $spoolfile = &get_unique_spool();
+		open SPOOL, ">" . $spoolfile;
+		print SPOOL $data;
+		system("copy /B $spoolfile $self->{'printer'}{$OSNAME}");
+
+		# place filename in command
+		my $cmd = $self->{print_command}->{linux}->{command};
+		
+		# print
+		system($cmd) or 
+		    Carp::croak "Can't execute print command: $cmd, $!\n"; 
+		
+		unlink $spoolfile;
+	    }
 
-    # windows
-    elsif ($self->{'system'} eq "MSWin32") {
-	# see which windows platform this is being run on.
-	my %win_info = GetWinVersion();
-	
-	# Windows NT (tested on NT4)
-	if ($win_info{Platform} eq "Win32_NT") {
-	    open SPOOL, ">>" . $self->{'printer'}{$OSNAME} or
-	      Carp::croak "Can't open print spool $self->{'printer'}: $!" ;
-	    print SPOOL $data or 
-	      Carp::croak "Can't write to print spool $self->{'printer'}: $!";
-	    close SPOOL;
-	} 
-	
-	# windows 9X
-	elsif ($win_info{Platform} =~ /^Win32_9/) {
-	    my $spoolfile = get_unique_spool();
-	    open SPOOL, ">" . $spoolfile;
-	    print SPOOL $data;
-	    system("copy /B $spoolfile $self->{'printer'}{$OSNAME}");
-	    unlink $spoolfile;
 	}
-    } #end windows
-}
+    } # end linux ############################################################
+
+
+    # windows ################################################################
+    elsif ($self->{'system'} eq "MSWin32") {
+	
+	unless ($self->{print_command}) {
+	    # default pipish method
+
+	    # Windows NT (tested on NT4)
+	    if (Win32::IsWinNT() ) {
+		open SPOOL, ">>" . $self->{'printer'}{$OSNAME} or
+		    Carp::croak "Can't open print spool $self->{'printer'}{$OSNAME}: $!" ;
+		print SPOOL $data or 
+		    Carp::croak "Can't write to print spool $self->{'printer'}: $!";
+		close SPOOL;
+	    } 
+	    
+	    # any other windows version
+	    else {
+		my $spoolfile = get_unique_spool();
+		open SPOOL, ">" . $spoolfile;
+		print SPOOL $data;
+		close SPOOL;
+		system("copy /B $spoolfile $self->{printer}{$OSNAME}");
+		unlink $spoolfile;
+	    }
+
+	} else {
+	    # custom print command
+	    if ($self->{print_command}->{MSWin32}->{type} eq 'file') {
+		# non-pipe accepting command - use a spoolfile
+		my $cmd = $self->{print_command}->{MsWin32}->{command};
+		my $spoolfile = get_unique_spool();
+		open SPOOL, ">" . $spoolfile;
+		print SPOOL $data;
+		system("$cmd") || Carp::croak $OS_ERROR;
+		unlink $spoolfile;
+	    } else {
+		# pipe accepting command
+		# can't use this - windows perl doesn't support pipes.
+	    }
+	}
+    } # end windows
+} 
 #############################################################################
 sub list_jobs {
     # list the current print queue
@@ -278,17 +344,17 @@ sub list_jobs {
     # linuxish systems
     if ($self->{'system'} eq "linux") {
 	# use available query program, lpq preferred      # DWP
-        my $lpcmd;                                     # DWP
-        if ( exists $self->{'program'}{'lpq'} ) {      # DWP
-            $lpcmd = $self->{'program'}{'lpq'}.' -P'   # DWP
-        } elsif ( exists $self->{'program'}{'lpstat'} ) { # DWP
-            $lpcmd = $self->{'program'}{'lpstat'}.' -o' # DWP
-        } else {                                       # DWP
-            Carp::croak "Can't find lpq or lpstat prog for jobs function" # DWP
-        }                                              # DWP
+	my $lpcmd;                                     # DWP
+	if ( exists $self->{'program'}{'lpq'} ) {      # DWP
+	    $lpcmd = $self->{'program'}{'lpq'}.' -P'   # DWP
+	    } elsif ( exists $self->{'program'}{'lpstat'} ) { # DWP
+		$lpcmd = $self->{'program'}{'lpstat'}.' -o' # DWP
+		} else {                                       # DWP
+		    Carp::croak "Can't find lpq or lpstat prog for jobs function" # DWP
+		    }                                              # DWP
 	my @lpq = `$lpcmd$self->{'printer'}{$OSNAME}`;     # DWP-use lpcmd for lpq/lpstat
 	chomp @_;
-        # lprng returns
+	# lprng returns
 	# Printer: lp@localhost 'HP Laserjet 4Si' (dest raw1@192.168.1.5)
 	# Queue: 1 printable job
 	# Server: pid 7145 active
@@ -298,77 +364,77 @@ sub list_jobs {
 	
 	my $pr = $self->{'printer'}{$OSNAME};                             
 
-    if ($lpq[0] =~ /^Printer/) {              # DWP - said queue, should be lpq
-	# first line of lpq starts with Printer
-	# lprng spooler, skip first 5 lines
+	if ($lpq[0] =~ /^Printer/) {              # DWP - said queue, should be lpq
+	    # first line of lpq starts with Printer
+	    # lprng spooler, skip first 5 lines
 	    for (my $i = 5; $i < @lpq; ++$i) {
-        push @queue, join(' ',(split(/\s+/,$lpq[$i]))[0,1,3..5]);         # DWP - fix to exclude class
+		push @queue, join(' ',(split(/\s+/,$lpq[$i]))[0,1,3..5]);         # DWP - fix to exclude class
 	    }
-    } elsif ($lpq[1] =~/^Rank/) {                      # DWP - said queue, should be lpq
-        # second line of BSD & solaris lpq starts with Rank                 DWP - compressed doc, inc solaris
-        # Rank   Owner   Job  Files        Total Size                       DWP - compressed doc
-        # active mwf     31   thesis.txt   682048 bytes                     DWP - compressed doc
-        for (my $i = 2; $i < @lpq; ++$i) {
+	} elsif ($lpq[1] =~/^Rank/) {                      # DWP - said queue, should be lpq
+	    # second line of BSD & solaris lpq starts with Rank                 DWP - compressed doc, inc solaris
+	    # Rank   Owner   Job  Files        Total Size                       DWP - compressed doc
+	    # active mwf     31   thesis.txt   682048 bytes                     DWP - compressed doc
+	    for (my $i = 2; $i < @lpq; ++$i) {
 		push @queue, $lpq[$i];
 	    }
-    } elsif ($lpq[0] =~ /^$pr-\d+\s+/ and $lpq[1] =~ / bytes/) {          # DWP
-        # hpux lpstat -o has multi-line entries                             DWP
-        #NE1-9638            da0240         priority 0  Mar 14 14:53 on NE1 DWP
-        #        (standard input)                          661 bytes        DWP
-        #NE1-110             ss0300         priority 0  Oct 19 12:51        DWP
-        #        mediafas             [ 3 copies ]       69 bytes           DWP
-        #        rescan               [ 3 copies ]       62 bytes           DWP
-        my @job;                                                          # DWP
-        foreach my $line ( @lpq ) {                                       # DWP
-            if ( $line =~ /^($pr-\d+)\s+(\S+)\s+priority/ ) {             # DWP
-                if ( @job ) {                             # previous job    DWP
-                    push @queue, join(' ',@job);                          # DWP
-                    @job<5 and Carp::cluck "Short job entry: $queue[-1] ";# DWP
-                }                                                         # DWP
-                @job = ( 'active', $2, $1 );              # rank,owner,job  DWP
-            } elsif ( $line =~ /^\s*(\S+|\(.+\))\s.*\s(\d+)\s+bytes/ ) {  # DWP
-                $job[3] = $job[3] ? $job[3].",$1" : $1;   # add file(s)     DWP
-                my $sz = $2;                                              # DWP
-                $line =~ /\s(\d+)\s+copies/ and ( $sz *= $1 ); # copies?    DWP
-                $job[4] = $job[4] ? $job[4].",$sz" : $sz; # add size(s)     DWP
-                $job[3] =~ s/ /_/g;                       # elim spaces     DWP
-            }                                                             # DWP
-        }                                                                 # DWP
-    } elsif ( ($lpq[1] !~ /\S/) and ($lpq[2] =~/^Rank/) ) {               # DWP
-        # third line of dec_osf lpq starts with Rank, second is blank       DWP
-        #Rank   Owner      Job  Files                        Total Size     DWP
-        #active ss0300     40   lpr.doc, Printer.pm          103014 bytes   DWP
-        #active ss0300     42   (standard input)             54585 bytes    DWP
-        for (my $i = 3; $i < @lpq; ++$i) {                                # DWP
-            $lpq[$i] =~ s/,\s/,/g;                        # multi-files     DWP
-            if ( $lpq[$i] =~ /(\(.*\))/ ) {               # spaces in file  DWP
-                my ($ofil,$nfil) = ($1,$1);                               # DWP
-                $nfil =~ s/ /_/g;                                         # DWP
-                ($ofil,$nfil) = (quotemeta($ofil),quotemeta($nfil));      # DWP
-                $lpq[$i] = s/$ofil/$nfil/;                                # DWP
-            }                                                             # DWP
-            push @queue, $lpq[$i];                                        # DWP
-        }
-    }
+	} elsif ($lpq[0] =~ /^$pr-\d+\s+/ and $lpq[1] =~ / bytes/) {          # DWP
+	    # hpux lpstat -o has multi-line entries                             DWP
+	    #NE1-9638            da0240         priority 0  Mar 14 14:53 on NE1 DWP
+	    #        (standard input)                          661 bytes        DWP
+	    #NE1-110             ss0300         priority 0  Oct 19 12:51        DWP
+	    #        mediafas             [ 3 copies ]       69 bytes           DWP
+	    #        rescan               [ 3 copies ]       62 bytes           DWP
+	    my @job;                                                          # DWP
+	    foreach my $line ( @lpq ) {                                       # DWP
+		if ( $line =~ /^($pr-\d+)\s+(\S+)\s+priority/ ) {             # DWP
+		    if ( @job ) {                             # previous job    DWP
+			push @queue, join(' ',@job);                          # DWP
+			@job<5 and Carp::cluck "Short job entry: $queue[-1] ";# DWP
+			}                                                         # DWP
+		    @job = ( 'active', $2, $1 );              # rank,owner,job  DWP
+		} elsif ( $line =~ /^\s*(\S+|\(.+\))\s.*\s(\d+)\s+bytes/ ) {  # DWP
+		    $job[3] = $job[3] ? $job[3].",$1" : $1;   # add file(s)     DWP
+		    my $sz = $2;                                              # DWP
+		    $line =~ /\s(\d+)\s+copies/ and ( $sz *= $1 ); # copies?    DWP
+		    $job[4] = $job[4] ? $job[4].",$sz" : $sz; # add size(s)     DWP
+		    $job[3] =~ s/ /_/g;                       # elim spaces     DWP
+		}                                                             # DWP
+	    }                                                                 # DWP
+	} elsif ( ($lpq[1] !~ /\S/) and ($lpq[2] =~/^Rank/) ) {               # DWP
+	    # third line of dec_osf lpq starts with Rank, second is blank       DWP
+	    #Rank   Owner      Job  Files                        Total Size     DWP
+	    #active ss0300     40   lpr.doc, Printer.pm          103014 bytes   DWP
+	    #active ss0300     42   (standard input)             54585 bytes    DWP
+	    for (my $i = 3; $i < @lpq; ++$i) {                                # DWP
+		$lpq[$i] =~ s/,\s/,/g;                        # multi-files     DWP
+		if ( $lpq[$i] =~ /(\(.*\))/ ) {               # spaces in file  DWP
+		    my ($ofil,$nfil) = ($1,$1);                               # DWP
+		    $nfil =~ s/ /_/g;                                         # DWP
+		    ($ofil,$nfil) = (quotemeta($ofil),quotemeta($nfil));      # DWP
+		    $lpq[$i] = s/$ofil/$nfil/;                                # DWP
+		}                                                             # DWP
+		push @queue, $lpq[$i];                                        # DWP
+	    }
+	}
 
 	# make the queue into an array of hashes
 	for(my $i = 0; $i < @queue; ++$i) {
 	    $queue[$i] =~ s/\s+/ /g; # remove extraneous spaces
 	    my @job = split / /, $queue[$i];
 	    $queue[$i] = ('Rank' => $job[0],
-                     'Owner' => $job[1],
-                       'Job' => $job[2],
-                     'Files' => $job[3],
-                      'Size' => $job[4]
-                     );
+			  'Owner' => $job[1],
+			  'Job' => $job[2],
+			  'Files' => $job[3],
+			  'Size' => $job[4]
+			  );
 	}
 
     } # end linux
     
     elsif ($self->{'system'} eq "MSWin32") {
 	# return an empty queue (for compatibility)
-      Carp::croak 'list_jobs  hasn\'t yet been written for windows. Share and enjoy';
-    }
+	Carp::croak 'list_jobs  hasn\'t yet been written for windows. Share and enjoy';
+      }
     return @queue;
 }
 #############################################################################
@@ -386,11 +452,18 @@ This version includes working support for Windows 95.
 =head1 SYNOPSIS
     
  use Printer;
-    
+
  $prn = new Printer('linux' => 'lp', 
-	            'MSWin32' => 'LPT1', 
+	 	    'MSWin32' => 'LPT1', 
 		    $OSNAME => 'Printer');
 
+ $prn->print_command('linux' = {'type' => 'pipe',
+			        'command' => 'lpr -P lp'},
+		    'MSWin32' = {'type' => 'command',
+				 'command' => 'gswin32c -sDEVICE=mswinpr2 
+                                 -dNOPAUSE -dBATCH $spoolfile'}
+		    );
+
  %available_printers = $prn->list_printers;
 
  $prn->use_default;
@@ -427,7 +500,7 @@ of C<$OSNAME> or C<$^O> and the corrections.
 
  $printer = new Printer('osname' => 'printer port');
  $printer = new Printer('MSWin32' => 'LPT1', 
-			'Linux' => 'lp');
+                        'Linux' => 'lp');
 
 This method takes a hash to set the printer
 name to be used for each operating system that this module is to
@@ -435,8 +508,39 @@ be used on (the hash keys are the values of $^O or $OSNAME for
 each platform) and returns a printer handle which 
 is used by the other methods.
 
+If you intend to use the C<use_default()> or C<print_command()> methods,
+you don't need to supply any parameters to C<new()>.
+
 This method dies with an error message on unsupported platforms.
 
+=head2 Define a printer command to use
+
+ $prn->print_command('linux' = {'type' => 'pipe',
+                     'command' => 'lpr -P lp'},
+                     'MSWin32' = {'type' => 'file',
+                                  'command' => 'gswin32c -sDEVICE=mswinpr2 
+                                  -dNOPAUSE -dBATCH $spoolfile'}
+                    );
+
+This method allows you to specify your own print command to use. It
+takes 2 parameters for each operating system: 
+
+=head3 type
+
+=over 4
+
+=item * pipe - the specified print command accepts data on a pipe.
+
+=item * file - the specified print command works on a file. The
+Printer module replaces $spoolfile with a temporary filename which contains
+the data to be printed 
+
+=back
+
+=head3 command
+
+This specifies the command to be used. 
+
 =head2 Select the default printer
 
  $printer->use_default;
@@ -451,12 +555,12 @@ this happens.
 
 =head3 Win32
 
-THe default printer is read from the registry (trust me, this works).
+ THe default printer is read from the registry (trust me, this works).
 
 =head2 List available printers
-     
- %printers = list_printers().
     
+ %printers = list_printers().
+  
 This returns a hash of arrays listing all available printers. 
 The hash keys are:
 
@@ -492,10 +596,10 @@ the print jobs.
 
  @queue = list_jobs();
  foreach $ref (@queue) {
-     foreach $field (qw/Rank Owner Job Files Size/) {
-	 print $field, " = ", $$ref{$field}, " ";
-     }
-     print "\n";
+    foreach $field (qw/Rank Owner Job Files Size/) {
+        print $field, " = ", $$ref{$field}, " ";
+    }
+ print "\n";
  }
 
 =for html </PRE>
@@ -524,14 +628,30 @@ David W Phillips (ss0300@dfa.state.ny.us)
 
 =item * Make list_jobs work on windows.
 
-=item * Test and fully port to UNIX.
-
 =item * Port to MacOS.
 
 =back
 
 =head1 Changelog
 
+=head2 0.95
+
+=over 4
+
+=item * added support for user defined print commands.
+
+=back
+
+=head2 0.94c
+
+=over 4
+
+=item * removed unwanted dependency on Win32::AdminMisc
+
+=item * added support of user-defined print command
+
+=back
+
 =head2 0.94b
 
 =over 4
@@ -539,6 +659,8 @@ David W Phillips (ss0300@dfa.state.ny.us)
 =item * added documentation of the array input capabilities of the print() 
 method
 
+=item * windows installation fixed (for a while)
+
 =back
 
 =head2 0.94a
@@ -566,7 +688,7 @@ method
 =over 4
 
 =item * Checked and modified for dec_osf, solaris and HP/UX thanks to
-    data from David Phillips.
+data from David Phillips.
 
 =item * Several quoting errors fixed.
 
@@ -577,7 +699,7 @@ method
 =item * list_jobs returns an array of hashes 
 
 =item * list_printers exported into main namespace so it can be called
-    without an object handle (which it doesn't need anyway).
+without an object handle (which it doesn't need anyway).
 
 =back
 
@@ -586,7 +708,7 @@ method
 =over 4
 
 =item * Printing on windows 95 now uses a unique spoolfile which
-    will not overwrite an existing file.
+will not overwrite an existing file.
 
 =item * Documentation spruced up to look like a normal linux manpage.
 
